@@ -1,12 +1,19 @@
 from typing import Union, Dict, List, Tuple, Callable, Any
+from copy import deepcopy
 import torch as t
 import torch.nn as nn
 import numpy as np
 
 from machin.model.nets.base import NeuralNetworkModule
 from machin.frame.buffers.buffer import Transition, Buffer
-from .base import TorchFramework
-from .utils import safe_call
+from .base import TorchFramework, Config
+from .utils import (
+    safe_call,
+    assert_and_get_valid_models,
+    assert_and_get_valid_optimizer,
+    assert_and_get_valid_criterion,
+    assert_and_get_valid_lr_scheduler,
+)
 
 
 class A2C(TorchFramework):
@@ -17,29 +24,34 @@ class A2C(TorchFramework):
     _is_top = ["actor", "critic"]
     _is_restorable = ["actor", "critic"]
 
-    def __init__(self,
-                 actor: Union[NeuralNetworkModule, nn.Module],
-                 critic: Union[NeuralNetworkModule, nn.Module],
-                 optimizer: Callable,
-                 criterion: Callable,
-                 *_,
-                 lr_scheduler: Callable = None,
-                 lr_scheduler_args: Tuple[Tuple, Tuple] = None,
-                 lr_scheduler_kwargs: Tuple[Dict, Dict] = None,
-                 actor_learning_rate: float = 0.001,
-                 critic_learning_rate: float = 0.001,
-                 critic_update_times: int = 50,
-                 entropy_weight: float = None,
-                 value_weight: float = 0.5,
-                 gradient_max: float = np.inf,
-                 gae_lambda: float = 1.0,
-                 discount: float = 0.99,
-                 replay_size: int = 500000,
-                 replay_device: Union[str, t.device] = "cpu",
-                 replay_buffer: Buffer = None,
-                 visualize: bool = False,
-                 visualize_dir: str = "",
-                 **__):
+    def __init__(
+        self,
+        actor: Union[NeuralNetworkModule, nn.Module],
+        critic: Union[NeuralNetworkModule, nn.Module],
+        optimizer: Callable,
+        criterion: Callable,
+        *_,
+        lr_scheduler: Callable = None,
+        lr_scheduler_args: Tuple[Tuple, Tuple] = None,
+        lr_scheduler_kwargs: Tuple[Dict, Dict] = None,
+        batch_size: int = 100,
+        actor_update_times: int = 5,
+        critic_update_times: int = 10,
+        actor_learning_rate: float = 0.001,
+        critic_learning_rate: float = 0.001,
+        entropy_weight: float = None,
+        value_weight: float = 0.5,
+        gradient_max: float = np.inf,
+        gae_lambda: float = 1.0,
+        discount: float = 0.99,
+        normalize_advantage: bool = True,
+        replay_size: int = 500000,
+        replay_device: Union[str, t.device] = "cpu",
+        replay_buffer: Buffer = None,
+        visualize: bool = False,
+        visualize_dir: str = "",
+        **__
+    ):
         """
         Important:
             when given a state, and an optional, action actor must
@@ -134,12 +146,13 @@ class A2C(TorchFramework):
             lr_scheduler_args: Arguments of the learning rate scheduler.
             lr_scheduler_kwargs: Keyword arguments of the learning
                 rate scheduler.
+            batch_size: Batch size used during training.
+            actor_update_times: Times to update actor in ``update()``.
+            critic_update_times: Times to update critic in ``update()``.
             actor_learning_rate: Learning rate of the actor optimizer,
                 not compatible with ``lr_scheduler``.
             critic_learning_rate: Learning rate of the critic optimizer,
                 not compatible with ``lr_scheduler``.
-            critic_update_times: Times to update your critic model in each
-                ``update()`` call.
             entropy_weight: Weight of entropy in your loss function, a positive
                 entropy weight will minimize entropy, while a negative one will
                 maximize entropy.
@@ -148,6 +161,7 @@ class A2C(TorchFramework):
             gae_lambda: :math:`\\lambda` used in generalized advantage
                 estimation.
             discount: :math:`\\gamma` used in the bellman function.
+            normalize_advantage: Whether to normalize the advantage function.
             replay_size: Replay buffer size. Not compatible with
                 ``replay_buffer``.
             replay_device: Device where the replay buffer locates on, Not
@@ -156,24 +170,27 @@ class A2C(TorchFramework):
             visualize: Whether visualize the network flow in the first pass.
             visualize_dir: Visualized graph save directory.
         """
+        self.batch_size = batch_size
+        self.actor_update_times = actor_update_times
+        self.critic_update_times = critic_update_times
         self.discount = discount
         self.value_weight = value_weight
         self.entropy_weight = entropy_weight
-        self.grad_max = gradient_max
+        self.gradient_max = gradient_max
         self.gae_lambda = gae_lambda
-        self.critic_upd_t = critic_update_times
+        self.normalize_advantage = normalize_advantage
         self.visualize = visualize
         self.visualize_dir = visualize_dir
 
         self.actor = actor
         self.critic = critic
-        self.actor_optim = optimizer(self.actor.parameters(),
-                                     lr=actor_learning_rate)
-        self.critic_optim = optimizer(self.critic.parameters(),
-                                      lr=critic_learning_rate)
-        self.replay_buffer = (Buffer(replay_size, replay_device)
-                              if replay_buffer is None
-                              else replay_buffer)
+        self.actor_optim = optimizer(self.actor.parameters(), lr=actor_learning_rate)
+        self.critic_optim = optimizer(self.critic.parameters(), lr=critic_learning_rate)
+        self.replay_buffer = (
+            Buffer(replay_size, replay_device)
+            if replay_buffer is None
+            else replay_buffer
+        )
 
         if lr_scheduler is not None:
             if lr_scheduler_args is None:
@@ -181,19 +198,29 @@ class A2C(TorchFramework):
             if lr_scheduler_kwargs is None:
                 lr_scheduler_kwargs = ({}, {})
             self.actor_lr_sch = lr_scheduler(
-                self.actor_optim,
-                *lr_scheduler_args[0],
-                **lr_scheduler_kwargs[0],
+                self.actor_optim, *lr_scheduler_args[0], **lr_scheduler_kwargs[0],
             )
             self.critic_lr_sch = lr_scheduler(
-                self.critic_optim,
-                *lr_scheduler_args[1],
-                **lr_scheduler_kwargs[1]
+                self.critic_optim, *lr_scheduler_args[1], **lr_scheduler_kwargs[1]
             )
 
         self.criterion = criterion
 
-        super(A2C, self).__init__()
+        super().__init__()
+
+    @property
+    def optimizers(self):
+        return [self.actor_optim, self.critic_optim]
+
+    @optimizers.setter
+    def optimizers(self, optimizers):
+        self.actor_optim, self.critic_optim = optimizers
+
+    @property
+    def lr_schedulers(self):
+        if hasattr(self, "actor_lr_sch") and hasattr(self, "critic_lr_sch"):
+            return [self.actor_lr_sch, self.critic_lr_sch]
+        return []
 
     def act(self, state: Dict[str, Any], *_, **__):
         """
@@ -206,10 +233,7 @@ class A2C(TorchFramework):
         # returned values is always more than one
         return safe_call(self.actor, state)
 
-    def _eval_act(self,
-                  state: Dict[str, Any],
-                  action: Dict[str, Any],
-                  *_, **__):
+    def _eval_act(self, state: Dict[str, Any], action: Dict[str, Any], *_, **__):
         """
         Use actor network to evaluate the log-likelihood of a given
         action in the current state.
@@ -235,10 +259,18 @@ class A2C(TorchFramework):
         Not suggested, since you will have to calculate "value" and "gae"
         by yourself.
         """
-        self.replay_buffer.append(transition, required_attrs=(
-            "state", "action", "next_state", "reward", "value",
-            "gae", "terminal"
-        ))
+        self.replay_buffer.append(
+            transition,
+            required_attrs=(
+                "state",
+                "action",
+                "next_state",
+                "reward",
+                "value",
+                "gae",
+                "terminal",
+            ),
+        )
 
     def store_episode(self, episode: List[Union[Transition, Dict]]):
         """
@@ -250,46 +282,59 @@ class A2C(TorchFramework):
 
         # calculate value for each transition
         for i in reversed(range(1, len(episode))):
-            episode[i - 1]["value"] = \
+            episode[i - 1]["value"] = (
                 episode[i]["value"] * self.discount + episode[i - 1]["reward"]
+            )
 
         # calculate advantage
         if self.gae_lambda == 1.0:
             for trans in episode:
-                trans["gae"] = (trans["value"] -
-                                self._criticize(trans["state"]).item())
+                trans["gae"] = trans["value"] - self._criticize(trans["state"]).item()
         elif self.gae_lambda == 0.0:
             for trans in episode:
-                trans["gae"] = (trans["reward"] +
-                                self.discount * (1 - float(trans["terminal"]))
-                                * self._criticize(trans["next_state"]).item() -
-                                self._criticize(trans["state"]).item())
+                trans["gae"] = (
+                    trans["reward"]
+                    + self.discount
+                    * (1 - float(trans["terminal"]))
+                    * self._criticize(trans["next_state"]).item()
+                    - self._criticize(trans["state"]).item()
+                )
         else:
             last_critic_value = 0
             last_gae = 0
             for trans in reversed(episode):
                 critic_value = self._criticize(trans["state"]).item()
-                gae_delta = (trans["reward"] +
-                             self.discount * last_critic_value
-                             * (1 - float(trans["terminal"])) -
-                             critic_value)
+                gae_delta = (
+                    trans["reward"]
+                    + self.discount * last_critic_value * (1 - float(trans["terminal"]))
+                    - critic_value
+                )
                 last_critic_value = critic_value
-                last_gae = trans["gae"] = (last_gae * self.discount
-                                           * (1 - float(trans["terminal"]))
-                                           * self.gae_lambda +
-                                           gae_delta)
+                last_gae = trans["gae"] = (
+                    last_gae
+                    * self.discount
+                    * (1 - float(trans["terminal"]))
+                    * self.gae_lambda
+                    + gae_delta
+                )
 
         for trans in episode:
-            self.replay_buffer.append(trans, required_attrs=(
-                "state", "action", "next_state", "reward", "value",
-                "gae", "terminal"
-            ))
+            self.replay_buffer.append(
+                trans,
+                required_attrs=(
+                    "state",
+                    "action",
+                    "next_state",
+                    "reward",
+                    "value",
+                    "gae",
+                    "terminal",
+                ),
+            )
 
-    def update(self,
-               update_value=True,
-               update_policy=True,
-               concatenate_samples=True,
-               **__):
+    def update(
+        self, update_value=True, update_policy=True, concatenate_samples=True, **__
+    ):
         """
         Update network weights by sampling from buffer. Buffer
         will be cleared after update is finished.
@@ -302,81 +347,86 @@ class A2C(TorchFramework):
         Returns:
             mean value of estimated policy value, value loss
         """
-        # sample a batch
-        batch_size, (state, action, reward, next_state,
-                     terminal, target_value, advantage) = \
-            self.replay_buffer.sample_batch(-1,
-                                            sample_method="all",
-                                            concatenate=concatenate_samples,
-                                            sample_attrs=[
-                                                "state", "action", "reward",
-                                                "next_state", "terminal",
-                                                "value", "gae"],
-                                            additional_concat_attrs=[
-                                                "value", "gae"
-                                            ])
-
-        # normalize advantage
-        advantage = ((advantage - advantage.mean()) /
-                     (advantage.std() + 1e-6))
-
-        if self.entropy_weight is not None:
-            __, action_log_prob, new_action_entropy, *_ = \
-                self._eval_act(state, action)
-        else:
-            __, action_log_prob, *_ = \
-                self._eval_act(state, action)
-            new_action_entropy = None
-
-        action_log_prob = action_log_prob.view(batch_size, 1)
-
-        # calculate policy loss
-        act_policy_loss = -(action_log_prob *
-                            advantage.to(action_log_prob.device))
-
-        if new_action_entropy is not None:
-            act_policy_loss += (self.entropy_weight *
-                                new_action_entropy.mean())
-
-        act_policy_loss = act_policy_loss.mean()
-
-        if self.visualize:
-            self.visualize_model(act_policy_loss, "actor",
-                                 self.visualize_dir)
-
-        # Update actor network
-        if update_policy:
-            self.actor.zero_grad()
-            act_policy_loss.backward()
-            nn.utils.clip_grad_norm_(
-                self.actor.parameters(), self.grad_max
-            )
-            self.actor_optim.step()
-
+        sum_act_loss = 0
         sum_value_loss = 0
-        for _ in range(self.critic_upd_t):
+        self.actor.train()
+        self.critic.train()
+        for _ in range(self.actor_update_times):
+            # sample a batch
+            batch_size, (state, action, advantage) = self.replay_buffer.sample_batch(
+                self.batch_size,
+                sample_method="random_unique",
+                concatenate=concatenate_samples,
+                sample_attrs=["state", "action", "gae"],
+                additional_concat_attrs=["gae"],
+            )
+
+            # normalize advantage
+            if self.normalize_advantage:
+                advantage = (advantage - advantage.mean()) / (advantage.std() + 1e-6)
+
+            if self.entropy_weight is not None:
+                __, action_log_prob, new_action_entropy, *_ = self._eval_act(
+                    state, action
+                )
+            else:
+                __, action_log_prob, *_ = self._eval_act(state, action)
+                new_action_entropy = None
+
+            action_log_prob = action_log_prob.view(batch_size, 1)
+
+            # calculate policy loss
+            act_policy_loss = -(action_log_prob * advantage.type_as(action_log_prob))
+
+            if new_action_entropy is not None:
+                act_policy_loss += self.entropy_weight * new_action_entropy.mean()
+
+            act_policy_loss = act_policy_loss.mean()
+            sum_act_loss += act_policy_loss.item()
+
+            if self.visualize:
+                self.visualize_model(act_policy_loss, "actor", self.visualize_dir)
+
+            # Update actor network
+            if update_policy:
+                self.actor.zero_grad()
+                self._backward(act_policy_loss)
+                nn.utils.clip_grad_norm_(self.actor.parameters(), self.gradient_max)
+                self.actor_optim.step()
+
+        for _ in range(self.critic_update_times):
+            # sample a batch
+            batch_size, (state, target_value) = self.replay_buffer.sample_batch(
+                self.batch_size,
+                sample_method="random_unique",
+                concatenate=concatenate_samples,
+                sample_attrs=["state", "value"],
+                additional_concat_attrs=["value"],
+            )
             # calculate value loss
             value = self._criticize(state)
-            value_loss = (self.criterion(target_value.to(value.device),
-                                         value) *
-                          self.value_weight)
+            value_loss = (
+                self.criterion(target_value.type_as(value), value) * self.value_weight
+            )
             sum_value_loss += value_loss.item()
 
             if self.visualize:
-                self.visualize_model(value_loss, "critic",
-                                     self.visualize_dir)
+                self.visualize_model(value_loss, "critic", self.visualize_dir)
 
             # Update critic network
             if update_value:
                 self.critic.zero_grad()
-                value_loss.backward()
-                nn.utils.clip_grad_norm_(
-                    self.critic.parameters(), self.grad_max
-                )
+                self._backward(value_loss)
+                nn.utils.clip_grad_norm_(self.critic.parameters(), self.gradient_max)
                 self.critic_optim.step()
 
         self.replay_buffer.clear()
-        return -act_policy_loss.item(), sum_value_loss / self.critic_upd_t
+        self.actor.eval()
+        self.critic.eval()
+        return (
+            -sum_act_loss / self.actor_update_times,
+            sum_value_loss / self.critic_update_times,
+        )
 
     def update_lr_scheduler(self):
         """
@@ -386,3 +436,69 @@ class A2C(TorchFramework):
             self.actor_lr_sch.step()
         if hasattr(self, "critic_lr_sch"):
             self.critic_lr_sch.step()
+
+    @classmethod
+    def generate_config(cls, config: Union[Dict[str, Any], Config]):
+        default_values = {
+            "models": ["Actor", "Critic"],
+            "model_args": ((), ()),
+            "model_kwargs": ({}, {}),
+            "optimizer": "Adam",
+            "criterion": "MSELoss",
+            "criterion_args": (),
+            "criterion_kwargs": {},
+            "lr_scheduler": None,
+            "lr_scheduler_args": None,
+            "lr_scheduler_kwargs": None,
+            "batch_size": 100,
+            "actor_update_times": 5,
+            "critic_update_times": 10,
+            "actor_learning_rate": 0.001,
+            "critic_learning_rate": 0.001,
+            "entropy_weight": None,
+            "value_weight": 0.5,
+            "gradient_max": np.inf,
+            "gae_lambda": 1.0,
+            "discount": 0.99,
+            "normalize_advantage": True,
+            "replay_size": 500000,
+            "replay_device": "cpu",
+            "replay_buffer": None,
+            "visualize": False,
+            "visualize_dir": "",
+        }
+        config = deepcopy(config)
+        config["frame"] = "A2C"
+        if "frame_config" not in config:
+            config["frame_config"] = default_values
+        else:
+            config["frame_config"] = {**config["frame_config"], **default_values}
+        return config
+
+    @classmethod
+    def init_from_config(
+        cls,
+        config: Union[Dict[str, Any], Config],
+        model_device: Union[str, t.device] = "cpu",
+    ):
+        f_config = deepcopy(config["frame_config"])
+        models = assert_and_get_valid_models(f_config["models"])
+        model_args = f_config["model_args"]
+        model_kwargs = f_config["model_kwargs"]
+        models = [
+            m(*arg, **kwarg).to(model_device)
+            for m, arg, kwarg in zip(models, model_args, model_kwargs)
+        ]
+        optimizer = assert_and_get_valid_optimizer(f_config["optimizer"])
+        criterion = assert_and_get_valid_criterion(f_config["criterion"])(
+            *f_config["criterion_args"], **f_config["criterion_kwargs"]
+        )
+        lr_scheduler = f_config["lr_scheduler"] and assert_and_get_valid_lr_scheduler(
+            f_config["lr_scheduler"]
+        )
+
+        f_config["optimizer"] = optimizer
+        f_config["criterion"] = criterion
+        f_config["lr_scheduler"] = lr_scheduler
+        frame = cls(*models, **f_config)
+        return frame
