@@ -1,9 +1,5 @@
 from multiprocessing import get_context, get_start_method
-from . import (
-    distributed, server, assigner,
-    exception, pickle, thread,
-    pool, queue
-)
+from . import distributed, server, assigner, exception, pickle, thread, pool, queue
 
 __all__ = [
     "get_context",
@@ -14,5 +10,5 @@ __all__ = [
     "exception",
     "pickle",
     "pool",
-    "queue"
+    "queue",
 ]
