@@ -8,21 +8,33 @@ from .helper_classes import Object
 
 class Config(Object):
     """
-    Attributes:
-        root_directory: (const) Root directory of your trials.
-        restart_from_trial: (const) Trial name (usually a date string)
-            to restart from.
+    A simple replacement for python dict.
     """
 
-    def __init__(self,
-                 root_directory: str = "./",
-                 restart_from_trial: str = None,
-                 **configs):
-        configs["root_directory"] = root_directory
-        configs["restart_from_trial"] = restart_from_trial
-        super(Config, self).__init__(configs,
-                                     const_attrs={"root_directory",
-                                                  "restart_from_trial"})
+    def __init__(self, **configs):
+        super().__init__(configs)
+
+    def get(self, key, default=None):
+        if key in self:
+            return self[key]
+        return default
+
+    def __iter__(self):
+        for key in self.__dict__:
+            if not key.startswith("__"):
+                yield key
+
+    def __contains__(self, key):
+        assert not key.startswith("__")
+        return hasattr(self, key)
+
+    def __getitem__(self, key):
+        assert not key.startswith("__")
+        return getattr(self, key)
+
+    def __setitem__(self, key, value):
+        assert not key.startswith("__")
+        setattr(self, key, value)
 
 
 def load_config_cmd(merge_conf: Config = None) -> Config:
@@ -69,13 +81,11 @@ def load_config_cmd(merge_conf: Config = None) -> Config:
     config_dict = {}
     if args.conf is not None:
         for env_str in args.conf:
-            name, value = env_str.split('=')
+            name, value = env_str.split("=")
             value = eval(value)
             config_dict[name] = value
 
-    return merge_config((Config()
-                         if merge_conf is None
-                         else merge_conf), config_dict)
+    return merge_config((Config() if merge_conf is None else merge_conf), config_dict)
 
 
 def load_config_file(json_file: str, merge_conf: Config = None) -> Config:
@@ -90,19 +100,17 @@ def load_config_file(json_file: str, merge_conf: Config = None) -> Config:
         configuration
     """
     # parse the configurations from the config json file provided
-    with open(json_file, 'r') as config_file:
+    with open(json_file) as config_file:
         config_dict = json.load(config_file)
 
-    return merge_config((Config()
-                         if merge_conf is None
-                         else merge_conf), config_dict)
+    return merge_config((Config() if merge_conf is None else merge_conf), config_dict)
 
 
 def save_config(conf: Config, json_file: str):
     """
     Dump config object to a json file.
     """
-    with open(json_file, 'w') as config_file:
+    with open(json_file, "w") as config_file:
         json.dump(conf.data, config_file, sort_keys=True, indent=4)
 
 
