@@ -8,7 +8,7 @@ import colorlog
 from logging import INFO
 
 
-class FakeLogger(object):
+class FakeLogger:
     def setLevel(self, level):
         pass
 
@@ -38,11 +38,13 @@ class FakeLogger(object):
 
 
 _default_handler = colorlog.StreamHandler()
-_default_handler.setFormatter(colorlog.ColoredFormatter(
-    "%(log_color)s[%(asctime)s] <%(levelname)s>:%(name)s:%(message)s"))
+_default_handler.setFormatter(
+    colorlog.ColoredFormatter(
+        "%(log_color)s[%(asctime)s] <%(levelname)s>:%(name)s:%(message)s"
+    )
+)
 
 default_logger = colorlog.getLogger("default_logger")
 default_logger.addHandler(_default_handler)
 default_logger.setLevel(INFO)
 fake_logger = FakeLogger()
-
